{***********************************************************

* Component: TRemainingTimeBar 1.0

* Platform: Delphi 2 (maybe D3, but i haven't tested yet)

* What is this? TRemainingTimeBar is a TProgressBar descendant
that provides the remaining time (in seconds) to complete
te bar.

* How does it work? It makes a projection based on the elapsed time
and the position to calculate the remaining time, no magic!

* Accuracy: Because TRemainingTimeBar rely on samples, the most
regular progress the most accurated result.

* Author: Gabriel Cherem <gabriel@projuris.com>

* Status: Freeware, fell free for suggestions.

* Disclaimer: Use for your count and risk, blah blah blah...
Since the source code is provided, if you find out a bug, you
can fix it.  But I'd like to be reported about!

* Using TRemainingTimeBar: Drop a TRemainingTimeBar in your form
just like a TProgressBar.  Call the Start method before to start
to reset the time counter, then use the RemainingTime function
to retrieve the remaining time, and use this value in some TLabel
like this:

  MyLabel.Caption := Format('%d minutes left...',[RemainingTimeBar.RemainingTime div 60]);

***********************************************************}



unit RemTimeBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;

type
  TRemainingTimeBar = class(TProgressBar)
  private
    { Private declarations }
    StartTime: Double;
  protected
    { Protected declarations }
  public
    { Public declarations }
    function RemainingTime: LongInt;
    procedure Start;
  published
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Win95', [TRemainingTimeBar]);
end;

procedure TRemainingTimeBar.Start;
begin
  StartTime := Time;
end;

function TRemainingTimeBar.RemainingTime: LongInt;
var
  Elapsed: Double;
begin
  Elapsed := (Time - StartTime) / 1.15740E-5;  // in seconds
  if Elapsed = 0 then
    Result := -1
  else
    Result := Round(Elapsed / (Position-Min) * (Max-Position-Min)) ;
end;

end.
